"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const result_type_finder_1 = require("./result-type-finder");
const findEntityForDiff = (propertyName) => {
    return propertyName.includes('xProperties') ? 'oad.unclassified' : `oad.${propertyName}`;
};
const createValidationResult = (options) => {
    const entity = findEntityForDiff(options.propertyName);
    return {
        action: options.action,
        destinationSpecEntityDetails: createSpecEntityDetails(options.destinationObject),
        entity,
        source: 'openapi-diff',
        sourceSpecEntityDetails: createSpecEntityDetails(options.sourceObject),
        type: result_type_finder_1.resultTypeFinder.lookup(entity, options.action)
    };
};
const createSpecEntityDetails = (parsedProperty) => {
    return parsedProperty
        ? {
            location: parsedProperty.originalPath.join('.'),
            pathMethod: null,
            pathName: null,
            value: parsedProperty.value
        }
        : {
            location: undefined,
            pathMethod: null,
            pathName: null,
            value: undefined
        };
};
const isDefined = (target) => {
    return !_.isUndefined(target);
};
const isDefinedDeep = (objectWithValue) => {
    return isDefined(objectWithValue) && isDefined(objectWithValue.value);
};
const isUndefinedDeep = (objectWithValue) => {
    return _.isUndefined(objectWithValue) || _.isUndefined(objectWithValue.value);
};
const findAdditionDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const isAddition = isUndefinedDeep(sourceObject) && isDefinedDeep(destinationObject);
    if (isAddition) {
        return [createValidationResult({ sourceObject, destinationObject, propertyName, action: 'add' })];
    }
    return [];
};
const findDeletionDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const isDeletion = isDefinedDeep(sourceObject) && isUndefinedDeep(destinationObject);
    if (isDeletion) {
        return [createValidationResult({ sourceObject, destinationObject, propertyName, action: 'delete' })];
    }
    return [];
};
const findEditionDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const isEdition = isDefinedDeep(sourceObject)
        && isDefinedDeep(destinationObject) && (sourceObject.value !== destinationObject.value);
    if (isEdition) {
        return [createValidationResult({ sourceObject, destinationObject, propertyName, action: 'edit' })];
    }
    return [];
};
const findDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const additionDiffs = findAdditionDiffsInProperty(sourceObject, destinationObject, propertyName);
    const deletionDiffs = findDeletionDiffsInProperty(sourceObject, destinationObject, propertyName);
    const editionDiffs = findEditionDiffsInProperty(sourceObject, destinationObject, propertyName);
    return _.concat([], additionDiffs, deletionDiffs, editionDiffs);
};
const isValueInArray = (object, array) => {
    return _.some(array, { value: object.value });
};
const findAdditionDiffsInArray = (sourceArrayContent, destinationArrayContent, arrayName) => {
    const arrayContentAdditionDiffs = _(destinationArrayContent)
        .filter((entry) => {
        return !isValueInArray(entry, sourceArrayContent);
    })
        .map((addedEntry) => {
        return createValidationResult({
            action: 'item.add',
            destinationObject: addedEntry,
            propertyName: arrayName,
            sourceObject: undefined
        });
    })
        .flatten()
        .value();
    return arrayContentAdditionDiffs;
};
const findDeletionDiffsInArray = (sourceArrayContent, destinationArrayContent, arrayName) => {
    const arrayContentDeletionDiffs = _(sourceArrayContent)
        .filter((entry) => {
        return !isValueInArray(entry, destinationArrayContent);
    })
        .map((deletedEntry) => {
        return createValidationResult({
            action: 'item.delete',
            destinationObject: undefined,
            propertyName: arrayName,
            sourceObject: deletedEntry
        });
    })
        .flatten()
        .value();
    return arrayContentDeletionDiffs;
};
const findDiffsInArray = (sourceArray, destinationArray, objectName) => {
    const arrayAdditionDiffs = findAdditionDiffsInProperty(sourceArray, destinationArray, objectName);
    const arrayDeletionDiffs = findDeletionDiffsInProperty(sourceArray, destinationArray, objectName);
    let arrayContentAdditionDiffs = [];
    if (!arrayAdditionDiffs.length) {
        const sourceArrayContent = sourceArray.value;
        const destinationArrayContent = destinationArray.value;
        arrayContentAdditionDiffs = findAdditionDiffsInArray(sourceArrayContent, destinationArrayContent, objectName);
    }
    let arrayContentDeletionDiffs = [];
    if (!arrayDeletionDiffs.length) {
        const sourceArrayContent = sourceArray.value;
        const destinationArrayContent = destinationArray.value;
        arrayContentDeletionDiffs = findDeletionDiffsInArray(sourceArrayContent, destinationArrayContent, objectName);
    }
    return _.concat([], arrayAdditionDiffs, arrayDeletionDiffs, arrayContentAdditionDiffs, arrayContentDeletionDiffs);
};
const findDiffsInXProperties = (sourceParsedXProperties, destinationParsedXProperties, xPropertyContainerName) => {
    const xPropertyUniqueNames = _(_.keys(sourceParsedXProperties))
        .concat(_.keys(destinationParsedXProperties))
        .uniq()
        .value();
    const xPropertyDiffs = _(xPropertyUniqueNames)
        .map((xPropertyName) => {
        return findDiffsInProperty(sourceParsedXProperties[xPropertyName], destinationParsedXProperties[xPropertyName], `${xPropertyContainerName}.${xPropertyName}`);
    })
        .flatten()
        .value();
    return xPropertyDiffs;
};
const findDiffsInSpecs = (parsedSourceSpec, parsedDestinationSpec) => {
    const infoDiffs = _.concat([], findDiffsInProperty(parsedSourceSpec.info.termsOfService, parsedDestinationSpec.info.termsOfService, 'info.termsOfService'), findDiffsInProperty(parsedSourceSpec.info.description, parsedDestinationSpec.info.description, 'info.description'), findDiffsInProperty(parsedSourceSpec.info.contact.name, parsedDestinationSpec.info.contact.name, 'info.contact.name'), findDiffsInProperty(parsedSourceSpec.info.contact.email, parsedDestinationSpec.info.contact.email, 'info.contact.email'), findDiffsInProperty(parsedSourceSpec.info.contact.url, parsedDestinationSpec.info.contact.url, 'info.contact.url'), findDiffsInProperty(parsedSourceSpec.info.license.name, parsedDestinationSpec.info.license.name, 'info.license.name'), findDiffsInProperty(parsedSourceSpec.info.license.url, parsedDestinationSpec.info.license.url, 'info.license.url'), findDiffsInProperty(parsedSourceSpec.info.title, parsedDestinationSpec.info.title, 'info.title'), findDiffsInProperty(parsedSourceSpec.info.version, parsedDestinationSpec.info.version, 'info.version'), findDiffsInXProperties(parsedSourceSpec.info.xProperties, parsedDestinationSpec.info.xProperties, 'info.xProperties'));
    const basePathDiffs = findDiffsInProperty(parsedSourceSpec.basePath, parsedDestinationSpec.basePath, 'basePath');
    const hostDiffs = findDiffsInProperty(parsedSourceSpec.host, parsedDestinationSpec.host, 'host');
    const openApiDiffs = findDiffsInProperty(parsedSourceSpec.openapi, parsedDestinationSpec.openapi, 'openapi');
    const schemesDiffs = findDiffsInArray(parsedSourceSpec.schemes, parsedDestinationSpec.schemes, 'schemes');
    const topLevelXPropertyDiffs = findDiffsInXProperties(parsedSourceSpec.xProperties, parsedDestinationSpec.xProperties, 'xProperties');
    return Promise.resolve(_.concat([], infoDiffs, basePathDiffs, hostDiffs, openApiDiffs, schemesDiffs, topLevelXPropertyDiffs));
};
exports.specDiffer = {
    diff: findDiffsInSpecs
};
